package com.lianqin360.aresrctbasic;

import com.facebook.react.LazyReactPackage;
import com.facebook.react.bridge.JavaScriptModule;
import com.facebook.react.bridge.ModuleSpec;
import com.facebook.react.bridge.NativeModule;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.module.model.ReactModuleInfoProvider;
import com.facebook.react.uimanager.ViewManager;
import com.lianqin360.aresrctbasic.jsmodules.WebViewJsModule;
import com.lianqin360.aresrctbasic.modules.ReactCameraManager;
import com.lianqin360.aresrctbasic.modules.ReactNativeMethodsManager;
import com.lianqin360.aresrctbasic.modules.cookiemanager.ReactCookieManager;
import com.lianqin360.aresrctbasic.modules.download.ReactDownloadManager;
import com.lianqin360.aresrctbasic.modules.restarter.ReactRestarter;
import com.lianqin360.aresrctbasic.views.IWebView.ReactWebViewLayoutManager;
import com.lianqin360.aresrctbasic.views.blurimage.BlurImageViewManager;
import com.lianqin360.aresrctbasic.views.camera.ReactCameraViewManager;
import com.lianqin360.aresrctbasic.views.loadingui.ReactCircleProgressBarViewManager;
import com.lianqin360.aresrctbasic.views.pdfView.PDFViewManager;
import com.lianqin360.aresrctbasic.views.picker.ReactPickerViewManager;
import com.lianqin360.aresrctbasic.views.switchView.ReactSwitchViewManager;
import com.lianqin360.aresrctbasic.views.video.ReactVideoViewManager;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.inject.Provider;

/**
 * Created by Spencer on 2017/6/2.
 */

public class AresBasicPackage extends LazyReactPackage{
    /**
     * @param reactContext react application context that can be used to create modules
     * @return list of module specs that can create the native modules
     */
    @Override
    public List<ModuleSpec> getNativeModules(final ReactApplicationContext reactContext) {
        return Arrays.asList(
                ModuleSpec.nativeModuleSpec(ReactCookieManager.class, new Provider<NativeModule>() {
                    @Override
                    public NativeModule get() {
                        return new ReactCookieManager(reactContext);
                    }
                }),
                ModuleSpec.nativeModuleSpec(ReactDownloadManager.class, new Provider<NativeModule>() {
                    @Override
                    public NativeModule get() {
                        return new ReactDownloadManager(reactContext);
                    }
                }),
                ModuleSpec.nativeModuleSpec(ReactRestarter.class, new Provider<NativeModule>() {
                    @Override
                    public NativeModule get() {
                        return new ReactRestarter(reactContext);
                    }
                }),
                ModuleSpec.nativeModuleSpec(ReactCameraManager.class, new Provider<NativeModule>() {
                    @Override
                    public NativeModule get() {
                        return new ReactCameraManager(reactContext);
                    }
                }),
                ModuleSpec.nativeModuleSpec(ReactNativeMethodsManager.class, new Provider<NativeModule>() {
                    @Override
                    public NativeModule get() {
                        return new ReactNativeMethodsManager(reactContext);
                    }
                })

        );
    }

    @Override
    public ReactModuleInfoProvider getReactModuleInfoProvider() {
        return LazyReactPackage.getReactModuleInfoProviderViaReflection(this);
    }
    @Override
    public List<ViewManager> createViewManagers(ReactApplicationContext reactContext) {
        List<ViewManager> viewManagers = new ArrayList<>();
        viewManagers.add(new BlurImageViewManager());
        viewManagers.add(new ReactCameraViewManager());
        viewManagers.add(new ReactVideoViewManager());
        viewManagers.add(new ReactWebViewLayoutManager());
        viewManagers.add(new ReactCircleProgressBarViewManager());
        viewManagers.add(new PDFViewManager());
        viewManagers.add(new ReactPickerViewManager());
        viewManagers.add(new ReactVideoViewManager());
        viewManagers.add(new ReactSwitchViewManager());
        viewManagers.add (new ReactWebViewLayoutManager());
        return viewManagers;
    }

}
