package com.lianqin360.aresrctbasic.modules;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Build;
import android.support.v4.app.ActivityCompat;
import android.util.Log;

import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.lianqin360.aresrctbasic.utils.ReactMediaHelper;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by Spencer on 16/7/28.
 */
public class ReactNativeMethodsManager extends ReactContextBaseJavaModule {
    private ReactMediaHelper mediaHelper;
    private ReactApplicationContext mReactContext;

    public ReactNativeMethodsManager(ReactApplicationContext reactContext) {

        super(reactContext);
        mReactContext = reactContext;
        mediaHelper = new ReactMediaHelper(reactContext);
    }

    /**
     * @return the name of this module. This will be the name used to {@code require()} this module
     * from javascript.
     */
    @Override
    public String getName() {
        return "NativeMethodsManager";
    }

    @ReactMethod
    public void openCamera(Callback callback) {
        mediaHelper.openCamera(callback);
    }

    @ReactMethod
    public void openVideo(Callback callback) {
        mediaHelper.openVideo(callback);
    }

    @ReactMethod
    public void openPhotoGallery(Callback callback) {
        mediaHelper.openPhotoGallery(callback);
    }

    @ReactMethod
    public void openVideoGallery(Callback callback) {
        mediaHelper.openVideoGallery(callback);
    }

    @ReactMethod
    public void callTo(String number) {
        Intent intent = new Intent(Intent.ACTION_CALL);
        intent.setData(Uri.parse("tel:" + number));
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        if (ActivityCompat.checkSelfPermission(mReactContext, Manifest.permission.CALL_PHONE) != PackageManager.PERMISSION_GRANTED) {
            // TODO: Consider calling
            //    ActivityCompat#requestPermissions
            // here to request the missing permissions, and then overriding
            //   public void onRequestPermissionsResult(int requestCode, String[] permissions,
            //                                          int[] grantResults)
            // to handle the case where the user grants the permission. See the documentation
            // for ActivityCompat#requestPermissions for more details.
            return;
        }
        mReactContext.startActivity(intent);
    }
    @ReactMethod
    public void mailTo(String address){
        Intent data=new Intent(Intent.ACTION_SENDTO);
        data.setData(Uri.parse("mailto:"+address));
        data.putExtra(Intent.EXTRA_SUBJECT, "");
        data.putExtra(Intent.EXTRA_TEXT, "");
        data.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        mReactContext.startActivity(data);
    }
    @ReactMethod
    public void isInstalled(String packageName,Callback callback){
        PackageManager pm = mReactContext.getPackageManager();
        boolean installed = false;
        try {
            pm.getPackageInfo(packageName, PackageManager.GET_ACTIVITIES);
            installed = true;
        } catch (PackageManager.NameNotFoundException e) {
            installed = false;
        }
        callback.invoke(installed);
    }
    @ReactMethod
    public void openApplication(String packageName){
        if(packageName.indexOf("://")!=-1){
            //通过url scheme打开
            Intent intent = new Intent();
            intent.setData(Uri.parse(packageName));
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            mReactContext.startActivity(intent);
        }else{
            Intent intent =  mReactContext.getPackageManager().getLaunchIntentForPackage(packageName);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            mReactContext.startActivity(intent);
        }
    }
    @ReactMethod
    public void openAppStore(String appUrl){
        String str = "market://details?id=" + appUrl;
        Intent intent = new Intent("android.intent.action.VIEW");
        intent.setData(Uri.parse(str));
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        mReactContext.startActivity(intent);
    }
    @Override
    public Map<String, Object> getConstants() {

        PackageManager manager = mReactContext.getPackageManager();
        PackageInfo info = null;
        int version_code;
        String version_name ;
        try {
            info = manager.getPackageInfo(mReactContext.getPackageName(), 0);
            version_code = info.versionCode;
            version_name = info.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
            version_code = 0;
            version_name = "";
        }

        Map envs = new HashMap();
        envs.put("VERSION_CODE",version_code);
        envs.put("VERSION_NAME",version_name);
        envs.put("devices",android.os.Build.MODEL);
        envs.put("SYS_VERSION", Build.VERSION.RELEASE);
        envs.put("DEVICE_NAME",android.os.Build.PRODUCT);
        envs.put("SYS_NAME",android.os.Build.VERSION.SDK_INT);
        return envs;
    }
}
