package com.lianqin360.aresrctbasic.utils;


import android.app.Activity;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.content.pm.LabeledIntent;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.DocumentsContract;
import android.provider.MediaStore;
import android.util.Log;

import com.facebook.react.bridge.ActivityEventListener;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.WritableMap;


import java.io.File;
import java.util.Random;

/**
 * Created by Spencer on 16/7/28.
 */
public class ReactMediaHelper implements ActivityEventListener {
    public static String SDCARD_PATH = Environment.getExternalStorageDirectory().getAbsolutePath();
    private final static int CAMERA = 100001;
    private final static int PHOTO_GALLERY = 100002;
    private final static int VIDEO = 100003;
    private final static int VIDEO_GALLERY = 100004;
    private ReactApplicationContext context;
    private Uri filePath;
    private Callback callback;

    public ReactMediaHelper(ReactApplicationContext context) {
        this.context = context;
        this.context.addActivityEventListener(this);
    }

    private Uri cFilePath() {
        return Uri.fromFile(new File(SDCARD_PATH + "/" + Environment.DIRECTORY_DCIM, String.valueOf(System.currentTimeMillis()) + ".jpg"));
    }

    public void openCamera() {
        Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
//        String fileName =EnvConfig.CACHE_PATH+"/temp"+String.valueOf(new Random().nextInt(10000)) +".jpg";
//        ContentValues values = new ContentValues();
//        values.put(MediaStore.Images.Media.DATA, fileName);
//        filePath = context.getContentResolver().insert(
//                MediaStore.Images.Media.EXTERNAL_CONTENT_URI, values);
        filePath = cFilePath();
        intent.putExtra(MediaStore.EXTRA_OUTPUT, filePath);
        this.context.startActivityForResult(intent, CAMERA, null);
    }

    public void openCamera(Callback callback) {
        this.callback = callback;
        openCamera();
    }

    public void openVideo() {
        Intent intent = new Intent(MediaStore.ACTION_VIDEO_CAPTURE);
        intent.putExtra(MediaStore.EXTRA_VIDEO_QUALITY, 1);//视频质量
        intent.putExtra(MediaStore.EXTRA_DURATION_LIMIT, 5 * 60);//视频时长
        filePath = Uri.fromFile(new File(SDCARD_PATH + "/" + Environment.DIRECTORY_DCIM, String.valueOf(System.currentTimeMillis()) + ".mp4"));
        intent.putExtra(MediaStore.EXTRA_OUTPUT, filePath);
        this.context.startActivityForResult(intent, VIDEO, null);
    }

    public void openVideo(Callback callback) {
        this.callback = callback;
        openVideo();
    }

    public void openPhotoGallery(Callback callback) {
        this.callback = callback;
        Intent intent = new Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
        this.context.startActivityForResult(intent, PHOTO_GALLERY, null);
    }

    private void onPhotoGalleryResult(int resultCode, Intent data) {
        WritableMap map = Arguments.createMap();
        if (resultCode != Activity.RESULT_OK) {
            map.putString("path", "");
        } else {
            map.putString("path", String.valueOf(data.getData()));
        }
        callback.invoke(false, map);

    }

    public void openVideoGallery(Callback callback) {
        this.callback = callback;
        Intent intent = new Intent(Intent.ACTION_PICK, MediaStore.Video.Media.EXTERNAL_CONTENT_URI);
        this.context.startActivityForResult(intent, VIDEO_GALLERY, null);
    }

    private void onVideoGalleryResult(int resultCode, Intent data) {
        WritableMap map = Arguments.createMap();
        if (resultCode != Activity.RESULT_OK) {
            map.putString("path", "");
        } else {
            String path = getFilePathFromContentUri(data.getData(),this.context.getContentResolver());
            map.putString("path", path);
        }
        callback.invoke(false, map);

    }

    private void onCameraResult(int resultCode, Intent data) {
        if (resultCode != Activity.RESULT_OK) {
            callback.invoke(true);
            return;
        } else {
            new Thread(new Runnable() {

                @Override
                public void run() {
                    // TODO Auto-generated method stub
                    WritableMap map = Arguments.createMap();
                    Uri uri = null;
                    String path = filePath.getPath();
                    Bitmap bitmap = BitmapHelper.compress(path, -1, 1024 * 1024);
                    String url = MediaStore.Images.Media.insertImage(
                            context.getContentResolver(),
                            bitmap, filePath.getQuery(),
                            filePath.getQuery());
                    map.putString("path", url);
                    callback.invoke(false, map);
                }
            }).start();
        }
    }

    private void onVideoResult(int resultCode, Intent data) {
        final WritableMap map = Arguments.createMap();
        if (resultCode != Activity.RESULT_OK) {
            map.putString("path", "");
            callback.invoke(true);
        } else {
            new Thread(new Runnable() {

                @Override
                public void run() {
                    // TODO Auto-generated method stub
                    String path = filePath.getPath();
                    map.putString("path", path);
                    callback.invoke(false, map);
                }
            }).start();
        }
    }

    @Override
    public void onActivityResult(Activity activity, int requestCode, int resultCode, Intent intent) {
        switch (requestCode) {
            case CAMERA:
                onCameraResult(resultCode, intent);
                break;
            case VIDEO:
                onVideoResult(resultCode, intent);
                break;
            case PHOTO_GALLERY:
                onPhotoGalleryResult(resultCode, intent);
                break;
            case VIDEO_GALLERY:
                onVideoGalleryResult(resultCode, intent);
                break;
            default:
                break;
        }
    }

    /**
     * Called when a new intent is passed to the activity
     *
     * @param intent
     */
    @Override
    public void onNewIntent(Intent intent) {

    }

    public static String getFilePathFromContentUri(Uri selectedVideoUri,
                                                   ContentResolver contentResolver) {
        String filePath;
        String[] filePathColumn = {MediaStore.MediaColumns.DATA};

        Cursor cursor = contentResolver.query(selectedVideoUri, filePathColumn, null, null, null);
//      也可用下面的方法拿到cursor
//      Cursor cursor = this.context.managedQuery(selectedVideoUri, filePathColumn, null, null, null);

        cursor.moveToFirst();

        int columnIndex = cursor.getColumnIndex(filePathColumn[0]);
        filePath = cursor.getString(columnIndex);
        cursor.close();
        return filePath;
    }

}
