package com.lianqin360.aresrctbasic.views.IWebView;

import android.content.Context;

import android.webkit.WebSettings;
import android.webkit.WebView;


import com.facebook.react.uimanager.ThemedReactContext;

/**
 * Created by Spencer on 16/6/15.
 */
public class MyWebView extends WebView {
    private ThemedReactContext context;
    protected MyWebViewDelegate delegate;
    //private CookieUtil cookieUtil;

    /**
     * WebView must be created with an context of the current activity
     *
     * Activity Context is required for creation of dialogs internally by WebView
     * Reactive Native needed for access to ReactNative internal system functionality
     *
     */
    public MyWebView(ThemedReactContext reactContext) {
        super(reactContext);
        this.context = reactContext;
        init();
    }
    private  void init(){
        setWebViewClient(new MyWebViewClient(this));
        setWebChromeClient(new MyWebChromeClient(this));
        setInitialScale(0);
        setVerticalScrollBarEnabled(false);

        WebSettings settings = getSettings();

        settings.setJavaScriptEnabled(true);
        //settings.setDefaultTextEncodingName("utf-8");
        settings.setJavaScriptCanOpenWindowsAutomatically(true);
        // settings.setLayoutAlgorithm(LayoutAlgorithm.NORMAL);
        settings.setSaveFormData(false);
        settings.setSavePassword(true);
        settings.setBlockNetworkImage(false);
        settings.setLoadsImagesAutomatically(true);

        // Jellybean rightfully tried to lock this down. Too bad they didn't
        // give us a whitelist
        // while we do this
		/*
		 * if (android.os.Build.VERSION.SDK_INT >=
		 * android.os.Build.VERSION_CODES.JELLY_BEAN) {
		 * settings.setAllowUniversalAccessFromFileURLs(true); } if
		 * (android.os.Build.VERSION.SDK_INT >=
		 * android.os.Build.VERSION_CODES.JELLY_BEAN_MR1) {
		 * settings.setMediaPlaybackRequiresUserGesture(false); }
		 */

        // Enable database
        // We keep this disabled because we use or shim to get around
        // DOM_EXCEPTION_ERROR_16
        String databasePath = this.getContext().getApplicationContext()
                .getDir("database", Context.MODE_PRIVATE).getPath();
        settings.setDatabaseEnabled(true);
        settings.setDatabasePath(databasePath);
        settings.setGeolocationDatabasePath(databasePath);

        // Enable DOM storage
        settings.setDomStorageEnabled(true);

        // Enable built-in geolocation
        settings.setGeolocationEnabled(true);

        // Enable AppCache
        // Fix for CB-2282
        settings.setAppCacheMaxSize(5 * 1048576);
        settings.setAppCachePath(databasePath);
        settings.setAppCacheEnabled(true);

        // Fix for CB-1405
        // Google issue 4641
        settings.getUserAgentString();
        settings.setAllowContentAccess(true);
        settings.setAllowFileAccess(true);
        settings.setAllowFileAccessFromFileURLs(true);

        // viewport
        settings.setUseWideViewPort(true);
        settings.setLoadWithOverviewMode(true);
        settings.setBuiltInZoomControls(false);
        settings.setSupportZoom(true);//
        settings.setSupportMultipleWindows(true);
        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.LOLLIPOP) {
            settings.setMixedContentMode(WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);
        }
    }

    public void setDelegate(MyWebViewDelegate delegate) {
        this.delegate = delegate;
    }

    public interface MyWebViewDelegate{
        void onProgressChanged(MyWebView view, int newProgress);
        void onPageFinished(MyWebView view, String url);
        void onPageStarted(MyWebView view, String url);
        void onShouldStartLoadWithRequest(MyWebView view, String url);
        void onInvoke(Object ... args);
    }

}
