package com.lianqin360.aresrctbasic.views.loadingui;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.RectF;
import android.util.AttributeSet;

import com.lianqin360.aresrctbasic.utils.MetricsHelper;
import com.lianqin360.aresrctbasic.views.animation.AnimationView;


/**
 * Created by Spencer on 16/9/19.
 */

public class CircleProgressBar extends AnimationView {

    private Context mContext;
    private Paint paintBackground;
    private Paint paintInnerCircle;
    private Paint paintOutterCircle;

    private int bacgroundColor = 0xffffffff;//默认白色
    private int innerCircleColor = 0xFFEE6723;//默认橘色
    private int outterCircleColor = 0xFFFEE5DD;//默认淡橘色

    private float process = 0;
    private int circleWidth;
    private int mesuredWidth;
    private int mesuredHeight;
    private int radius;
    private Point center;//圆心
    private RectF circleRectF;
    private int paddingLeft = 0;
    private int paddingRight = 0;
    private int paddingTop = 0;
    private int paddingBottom = 0;
    private boolean needRemeasure = true;
    private int v = 1;//速度
    private float currentProcess = 0;

    public CircleProgressBar(Context context) {
        super(context);
        initView(context);
    }
    public CircleProgressBar(Context context, AttributeSet attrs) {
        super(context, attrs);
    }

    public CircleProgressBar(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
    }

    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
        super.onSizeChanged(w, h, oldw, oldh);
        needRemeasure = true;
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        remeasure();
        drawBackground(canvas);
        drawOuterCircle(canvas);
        drawInnerCircle(canvas);
    }

    private void initView(Context context){
        mContext = context;

        circleWidth = MetricsHelper.dp2px(context,2);//默认进度条宽度2dp

        initPaints();


    }
    private void initPaints(){
        paintBackground = new Paint();
        paintBackground.setColor(bacgroundColor);
        paintBackground.setStyle(Paint.Style.FILL);
        paintBackground.setAntiAlias(true);

        paintInnerCircle = new Paint();
        paintInnerCircle.setColor(innerCircleColor);
        paintInnerCircle.setStrokeWidth(circleWidth);
        paintInnerCircle.setStyle(Paint.Style.STROKE);
        paintInnerCircle.setAntiAlias(true);


        paintOutterCircle = new Paint();
        paintOutterCircle.setColor(outterCircleColor);
        paintOutterCircle.setStrokeWidth(circleWidth);
        paintOutterCircle.setStyle(Paint.Style.STROKE);
        paintOutterCircle.setAntiAlias(true);

    }
    private void drawBackground(Canvas canvas){
        canvas.drawCircle(center.x,center.y,radius,paintBackground);
    }
    private void drawInnerCircle(Canvas canvas){
        canvas.drawArc(circleRectF,-90,(float) (currentProcess*3.6),false,paintInnerCircle);
    }
    private void drawOuterCircle(Canvas canvas){
        canvas.drawCircle(center.x,center.y,radius,paintOutterCircle);
    }
    private void remeasure(){
        if(needRemeasure){
            mesuredHeight = getHeight();
            mesuredWidth = getWidth();
            radius = Math.min(mesuredHeight-paddingBottom-paddingTop,mesuredWidth-paddingLeft-paddingRight)/2-circleWidth;

            center = new Point(mesuredWidth/2,mesuredHeight/2);

            circleRectF = new RectF(
                    center.x-radius,
                    center.y-radius,
                    center.x+radius,
                    center.y+radius);
        }
        needRemeasure = false;
    }

    public void setPaintBackground(Paint paintBackground) {
        this.paintBackground = paintBackground;
        invalidate();
    }

    public void setPaintInnerCircle(Paint paintInnerCircle) {
        this.paintInnerCircle = paintInnerCircle;
        invalidate();
    }

    public void setPaintOutterCircle(Paint paintOutterCircle) {
        this.paintOutterCircle = paintOutterCircle;
        invalidate();
    }

    public void setBacgroundColor(int bacgroundColor) {
        this.bacgroundColor = bacgroundColor;
        paintBackground.setColor(bacgroundColor);
        invalidate();
    }

    public void setInnerCircleColor(int innerCircleColor) {
        this.innerCircleColor = innerCircleColor;
        paintInnerCircle.setColor(innerCircleColor);
        invalidate();
    }

    public void setOutterCircleColor(int outterCircleColor) {
        this.outterCircleColor = outterCircleColor;
        paintOutterCircle.setColor(outterCircleColor);
        invalidate();
    }

    public void setProcess(float process) {
        this.process = process;
        currentProcess = 0;
        startAnimate();
    }

    public void setCircleWidth(int circleWidth) {
        this.circleWidth = circleWidth;
        needRemeasure = true;
        invalidate();
    }

    @Override
    public void onAnimate(long timestamp,long v) {
        invalidate();
    }

    @Override
    public long getNextValue() {
        if(currentProcess<process){
            currentProcess+=v;
        }else{
            currentProcess = process;
        }
        return (long)currentProcess;
    }

    @Override
    public boolean isFinish() {
        return currentProcess == process;
    }

}
