//
//  RCTTouchID.m
//  Touch
//
//  Created by sunlinlong on 16/7/4.
//  Copyright © 2016年 Facebook. All rights reserved.
//

#import "RCTTouchIDManager.h"
#import "WJTouchID.h"
#import "sys/utsname.h"
#import <UIKit/UIKit.h>

@interface RCTTouchIDManager ()<WJTouchIDDelegate>
@end

@implementation RCTTouchIDManager
{
  RCTResponseSenderBlock touchCallback;
}


RCT_EXPORT_MODULE(RCTTouchIDManager)

/**
 *  当前设备不支持指纹识别
 *
 *  (English Comments) The current device does not support fingerprint identification
 */
RCT_EXPORT_METHOD(isEnable:(RCTResponseSenderBlock)callback)
{
  if ([[self deviceVersion] isEqualToString:@"iPhone 1G" ]||
      [[self deviceVersion] isEqualToString:@"iPhone 3G"]||
      [[self deviceVersion] isEqualToString:@"iPhone 3GS"]||
      [[self deviceVersion] isEqualToString:@"iPhone 4"]||
      [[self deviceVersion] isEqualToString:@"Verizon iPhone 4"]||
      [[self deviceVersion]  isEqualToString:@"iiPhone 4S"]||
      [[self deviceVersion]  isEqualToString:@"iPhone 5"]||
      [[self deviceVersion]  isEqualToString:@"iPhone 5C"]
      ) {
    NSLog(@"当前设备不支持指纹识别");
    callback(@[@NO]);
  }else{
    NSLog(@"当前设备支持指纹识别");
    callback(@[@YES]);
  }
}

RCT_EXPORT_METHOD(prepare:(RCTResponseSenderBlock)callback)
{
    touchCallback = callback;
    
    WJTouchID *touchID = [[WJTouchID alloc]init];
    touchID.delegate = self;
    [touchID startWJTouchIDWithMessage:WJNotice(@"请在Home键上按上您的指纹",@"" ) fallbackTitle:@"" delegate:self];
}

-(void)WJTouchIDIsNotSupport{
  NSLog(@"当前设备不支持TouchID");
  touchCallback(@[@NO]);
}

/**
 *  TouchID验证成功
 *
 *  (English Comments) Authentication Successul  Authorize Success
 */
- (void) WJTouchIDAuthorizeSuccess {
  NSLog(@"登录成功");
  
  touchCallback(@[@YES]);
  
}

/**
 *  TouchID验证失败
 *
 *  (English Comments) Authentication Failure
 */
- (void) WJTouchIDAuthorizeFailure {
  NSLog(@"登录失败");
  touchCallback(@[@NO]);

}

/**
 *  设备版本
 *
 *  @return e.g. iPhone 5S
 */
- (NSString*)deviceVersion
{
  // 需要#import "sys/utsname.h"
  struct utsname systemInfo;
  uname(&systemInfo);
  NSString *deviceString = [NSString stringWithCString:systemInfo.machine encoding:NSUTF8StringEncoding];
  
  //iPhone
  if ([deviceString isEqualToString:@"iPhone1,1"])    return @"iPhone 1G";
  if ([deviceString isEqualToString:@"iPhone1,2"])    return @"iPhone 3G";
  if ([deviceString isEqualToString:@"iPhone2,1"])    return @"iPhone 3GS";
  if ([deviceString isEqualToString:@"iPhone3,1"])    return @"iPhone 4";
  if ([deviceString isEqualToString:@"iPhone3,2"])    return @"Verizon iPhone 4";
  if ([deviceString isEqualToString:@"iPhone4,1"])    return @"iPhone 4S";
  if ([deviceString isEqualToString:@"iPhone5,1"])    return @"iPhone 5";
  if ([deviceString isEqualToString:@"iPhone5,2"])    return @"iPhone 5";
  if ([deviceString isEqualToString:@"iPhone5,3"])    return @"iPhone 5C";
  if ([deviceString isEqualToString:@"iPhone5,4"])    return @"iPhone 5C";
  if ([deviceString isEqualToString:@"iPhone6,1"])    return @"iPhone 5S";
  if ([deviceString isEqualToString:@"iPhone6,2"])    return @"iPhone 5S";
  if ([deviceString isEqualToString:@"iPhone7,1"])    return @"iPhone 6 Plus";
  if ([deviceString isEqualToString:@"iPhone7,2"])    return @"iPhone 6";
  if ([deviceString isEqualToString:@"iPhone8,1"])    return @"iPhone 6s";
  if ([deviceString isEqualToString:@"iPhone8,2"])    return @"iPhone 6s Plus";
  
  return deviceString;
}

@end
