//
//  PhotoViewController.m
//  m
//
//  Created by Spencer on 16/7/28.
//  Copyright © 2016年 Facebook. All rights reserved.
//

#import "RCTMediaViewHelper.h"


@implementation RCTMediaViewHelper
{
  RCTResponseSenderBlock _callback;
    AVPlayer * _player;
}
-(void)openImagePickerByType:(UIImagePickerControllerSourceType)type{
    
    if ([UIImagePickerController isSourceTypeAvailable: type])
    {
        UIImagePickerController *picker = [[UIImagePickerController alloc] init];
        picker.delegate = self;
        //设置拍照后的图片可被编辑
        picker.allowsEditing = YES;
        picker.sourceType = type;
        dispatch_async(dispatch_get_main_queue(), ^{
            UIViewController *root = [[[[UIApplication sharedApplication] delegate] window] rootViewController];
            while (root.presentedViewController != nil) {
                root = root.presentedViewController;
            }
            [root presentViewController:picker animated:YES completion:nil];
        });
    }else{
        NSLog(@"模拟器中无法打开照相机,请在真机中使用");
    }
}

-(void)openImagePickerByVideo:(UIImagePickerControllerSourceType)type{
    
    
    if ([UIImagePickerController
         isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]) {
        NSArray *availableMediaTypes = [UIImagePickerController
                                        availableMediaTypesForSourceType:UIImagePickerControllerSourceTypeCamera];
        if ([availableMediaTypes containsObject:(NSString *)kUTTypeMovie]) {
            // 支持视频录制
            UIImagePickerController *picker = [[UIImagePickerController alloc] init];
            picker.sourceType=UIImagePickerControllerSourceTypeCamera;//设置image picker的来源，这里设置为摄像头
            
            picker.mediaTypes=@[(NSString *)kUTTypeMovie];
            /*
             UIImagePickerControllerQualityTypeHigh = 0,       // highest quality
             UIImagePickerControllerQualityTypeMedium = 1,     // medium quality, suitable for transmission via Wi-Fi
             UIImagePickerControllerQualityTypeLow = 2,         // lowest quality, suitable for tranmission via cellular network
             UIImagePickerControllerQualityType640x480 NS_ENUM_AVAILABLE_IOS(4_0) = 3,    // VGA quality
             UIImagePickerControllerQualityTypeIFrame1280x720 NS_ENUM_AVAILABLE_IOS(5_0) = 4,
             UIImagePickerControllerQualityTypeIFrame960x540 NS_ENUM_AVAILABLE_IOS(5_0) = 5,
             */
            picker.videoQuality=4;
//            picker.cameraCaptureMode=UIImagePickerControllerCameraCaptureModeVideo;//设置摄像头模式（拍照，录制视频）
            
            picker.videoMaximumDuration =300.0f;
            picker.allowsEditing=YES;//允许编辑
            picker.delegate=self;//设置代理，检测操作
            
            dispatch_async(dispatch_get_main_queue(), ^{
                UIViewController *root = [[[[UIApplication sharedApplication] delegate] window] rootViewController];
                while (root.presentedViewController != nil) {
                    root = root.presentedViewController;
                }
                [root presentViewController:picker animated:YES completion:nil];
            });
        }
    }else{
        NSLog(@"模拟器中无法打开照相机,请在真机中使用");
    }
}

-(NSString *)createName{
  return [NSString stringWithFormat:@"/image%d.jpg", arc4random() % 10000];
}
//开始拍照
-(void)openCamera:(RCTResponseSenderBlock) callback
{
    _callback = callback;
    UIImagePickerControllerSourceType sourceType = UIImagePickerControllerSourceTypeCamera;
    [self openImagePickerByType:sourceType];
    
}
//开始视频
-(void)openVideo:(RCTResponseSenderBlock) callback
{
    _callback = callback;
//    UIImagePickerControllerSourceType sourceType = UIImagePickerControllerSourceTypeCamera;
    [self openImagePickerByVideo:UIImagePickerControllerSourceTypeCamera];
    
}
-(void)openPhotoGallery;
{
    UIImagePickerControllerSourceType sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    [self openImagePickerByType:sourceType];
}
-(void)openVideoGallery;
{
    UIImagePickerController *picker = [[UIImagePickerController alloc] init];
    picker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    picker.mediaTypes = [[NSArray alloc] initWithObjects: (NSString *) kUTTypeMovie, nil];
    picker.delegate = self;
    dispatch_async(dispatch_get_main_queue(), ^{
        UIViewController *root = [[[[UIApplication sharedApplication] delegate] window] rootViewController];
        while (root.presentedViewController != nil) {
            root = root.presentedViewController;
        }
        [root presentViewController:picker animated:YES completion:nil];
    });

}

//打开本地相册
-(void)openPhotoGallery:(RCTResponseSenderBlock) callback;
{
    _callback = callback;
    [self openPhotoGallery];
}

//打开本地相册(视频)
-(void)openVideoGallery:(RCTResponseSenderBlock) callback;
{
    _callback = callback;
    [self openVideoGallery];
}

#pragma mark - UIImagePickerController代理方法
-(void)imagePickerController:(UIImagePickerController*)picker didFinishPickingMediaWithInfo:(NSDictionary *)info

{
  
  NSString *type = [info objectForKey:UIImagePickerControllerMediaType];
  
  //当选择的类型是图片
    if ([type isEqualToString:(NSString *)kUTTypeImage])
  {
    //先把图片转成NSData
    UIImage* image = [info objectForKey:UIImagePickerControllerEditedImage];
    NSData *data;
    if (UIImagePNGRepresentation(image) == nil)
    {
      data = UIImageJPEGRepresentation(image, 1.0);
    }
    else
    {
      data = UIImagePNGRepresentation(image);
    }
    
    //图片保存的路径
    //这里将图片放在沙盒的documents文件夹中
    NSString * DocumentsPath = [NSHomeDirectory() stringByAppendingPathComponent:@"Documents"];
    
    //文件管理器
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *fileName = [self createName];

    //把刚刚图片转换的data对象拷贝至沙盒中 并保存为image.png
    [fileManager createDirectoryAtPath:DocumentsPath withIntermediateDirectories:YES attributes:nil error:nil];
    
    [fileManager createFileAtPath:[DocumentsPath stringByAppendingString:fileName] contents:data attributes:nil];
    //得到选择后沙盒中图片的完整路径
    NSString *filePath = [[NSString alloc]initWithFormat:@"%@%@",DocumentsPath,  fileName];
    _callback(@[@NO,@{@"path":filePath}]);
    _callback = nil;
    
    
  }else if([type isEqualToString:(NSString *)kUTTypeMovie]){
      NSURL *filePath = [info objectForKey:UIImagePickerControllerMediaURL];//视频路径
      NSString *urlStr=[filePath path];
      if (UIVideoAtPathIsCompatibleWithSavedPhotosAlbum(urlStr)) {
          //保存视频到相簿，注意也可以使用ALAssetsLibrary来保存
          UISaveVideoAtPathToSavedPhotosAlbum(urlStr,self, @selector(video:didFinishSavingWithError:contextInfo:),nil);//保存视频到相簿
      }
      NSURL *sourceURL = [info objectForKey:UIImagePickerControllerMediaURL];
      NSURL *newVideoUrl ; //一般.mp4
      NSDateFormatter *formater = [[NSDateFormatter alloc] init];//用时间给文件全名，以免重复，在测试的时候其实可以判断文件是否存在若存在，则删除，重新生成文件即可
      [formater setDateFormat:@"yyyy-MM-dd-HH:mm:ss"];
      newVideoUrl = [NSURL fileURLWithPath:[NSHomeDirectory() stringByAppendingFormat:@"/Documents/output-%@.mp4", [formater stringFromDate:[NSDate date]]]] ;//这个是保存在app自己的沙盒路径里，后面可以选择是否在上传后删除掉。我建议删除掉，免得占空间。
      
      [self convertVideoQuailtyWithInputURL:sourceURL outputURL:newVideoUrl completeHandler:nil];
      
  }
    
    //关闭界面
    [picker dismissViewControllerAnimated:YES completion: (nil)];
    
  
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
  NSLog(@"您取消了选择图片");
  _callback(@[@YES,@{@"path":@""}]);
  [picker dismissViewControllerAnimated:YES completion: (nil)];
  _callback = nil;
}

- (void)video:(NSString *)videoPath didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInfo{
    if (error) {
        NSLog(@"保存视频过程中发生错误，错误信息:%@",error.localizedDescription);
    }else{
        NSLog(@"视频保存成功.");
    }
}
- (void) convertVideoQuailtyWithInputURL:(NSURL*)inputURL
                               outputURL:(NSURL*)outputURL
                         completeHandler:(void (^)(AVAssetExportSession*))handler
{
    AVURLAsset *avAsset = [AVURLAsset URLAssetWithURL:inputURL options:nil];
    
    AVAssetExportSession *exportSession = [[AVAssetExportSession alloc] initWithAsset:avAsset presetName:AVAssetExportPresetMediumQuality];
    //  NSLog(resultPath);
    exportSession.outputURL = outputURL;
    exportSession.outputFileType = AVFileTypeMPEG4;
    exportSession.shouldOptimizeForNetworkUse= YES;
    [exportSession exportAsynchronouslyWithCompletionHandler:^(void)
     {
         switch (exportSession.status) {
             case AVAssetExportSessionStatusCancelled:
                 NSLog(@"AVAssetExportSessionStatusCancelled");
                 break;
             case AVAssetExportSessionStatusUnknown:
                 NSLog(@"AVAssetExportSessionStatusUnknown");
                 break;
             case AVAssetExportSessionStatusWaiting:
                 NSLog(@"AVAssetExportSessionStatusWaiting");
                 break;
             case AVAssetExportSessionStatusExporting:
                 NSLog(@"AVAssetExportSessionStatusExporting");
                 break;
             case AVAssetExportSessionStatusCompleted:
                 NSLog(@"AVAssetExportSessionStatusCompleted");
                 [self uploadVideo:[outputURL path]];
//                 UISaveVideoAtPathToSavedPhotosAlbum([outputURL path], self, nil, NULL);//这个是保存到手机相册
//                 [[NSFileManager defaultManager] removeItemAtPath:[outputURL path] error:nil];//上传之后就删除，以免占用手机硬盘空间;
                 break;
             case AVAssetExportSessionStatusFailed:
                 NSLog(@"AVAssetExportSessionStatusFailed");
                 break;
         }
         
     }];
    
}

-(void)uploadVideo:(NSURL*)URL{
    NSString *filePath = [[NSString alloc]initWithFormat:URL];
    _callback(@[@NO,@{@"path":filePath}]);
    NSLog(@"==========>%@",filePath);
    _callback = nil;
}


@end


