//
//  CircleProcessBar.m
//  tbj
//
//  Created by Spencer on 16/9/19.
//  Copyright © 2016年 Facebook. All rights reserved.
//

#import "CircleProcessBar.h"

@implementation CircleProcessBar
{
    CADisplayLink *caDisplayLink;
    
    BOOL needRemeasure;
    int measuredWidth;
    int measuredHeight;
    int radius;
    CGPoint center;//圆心
    int paddingLeft;
    int paddingRight;
    int paddingTop;
    int paddingBottom;
    
    int currentProcess;
    int v;
}
-(void)setProcess:(int)process{
    _process = process;
    currentProcess = 0;
    //[self checkIsAnimatedFinish];
}
- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self initView];
    }
    return self;
}
-(void)drawRect:(CGRect)aRect{
    @try {
        [self remeasure];
        [self drawBackground:aRect];
        [self drawOutterCircle:aRect];
        [self drawInnerCircle:aRect];
    } @catch (NSException *exception) {
    } @finally {
    }
}

-(void) initView{
    needRemeasure = YES;
    
    currentProcess = 0;
    v = 2;
    
    paddingLeft = 0;
    paddingRight = 0;
    paddingTop = 0;
    paddingBottom = 0;
    
    _circleBackgroundColor = @"#FFFFFFFF";
    _innerCircleColor = @"#FFEE6723";
    _outterCircleColor = @"#FFFFFFFF";
    
    _circleWidth = 2;
    caDisplayLink = [CADisplayLink displayLinkWithTarget:self selector:@selector(checkIsAnimatedFinish)];
    [caDisplayLink addToRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
}
-(void)remeasure{
    if(needRemeasure){
        measuredWidth = self.bounds.size.width;
        measuredHeight = self.bounds.size.height;
        radius = MIN(measuredHeight-paddingBottom-paddingTop,measuredWidth-paddingLeft-paddingRight)/2-_circleWidth;
        center = CGPointMake(measuredWidth/2, measuredHeight/2);
        
    }
    needRemeasure = NO;
    
}

-(void)drawBackground:(CGRect)aRect{
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, [self getColor:_circleBackgroundColor].CGColor);
    CGContextAddArc(context, center.x, center.y, radius, 0, M_PI*2, true);
    CGContextFillPath(context);
    
}
-(void)drawInnerCircle:(CGRect)aRect{
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetStrokeColorWithColor(context,[self getColor:_innerCircleColor].CGColor );
    CGContextSetLineWidth(context, _circleWidth);
    CGContextAddArc(context, center.x, center.y, radius, -M_PI/2, (float)currentProcess/50*M_PI-M_PI/2, false);
    CGContextStrokePath(context);
    
}
-(void)drawOutterCircle:(CGRect)aRect{
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetStrokeColorWithColor(context,[self getColor:_outterCircleColor].CGColor );
    CGContextSetLineWidth(context, _circleWidth);
    CGContextAddArc(context, center.x, center.y, radius, 0, M_PI*2, true);
    CGContextStrokePath(context);
    
    
}
-(void)checkIsAnimatedFinish{
    if(currentProcess==_process){
        return;
    }
    [self getNextValue];
    [self setNeedsDisplay];
}
-(int)getNextValue{
    if(currentProcess<_process){
        currentProcess+=v;
    }else{
        currentProcess=_process;
    }
    return currentProcess;
}
//获取颜色
- (UIColor *)getColor:(NSString *)colorString {
    float red,green,blue,opacity;
    opacity = 1.0f;
    if([colorString hasPrefix:@"#"] == 1){
        NSString *string = [colorString substringFromIndex:1];//去掉#号
        switch (string.length) {
            case 3:
                red =(float)strtoul([[string substringWithRange:NSMakeRange(0, 1)] UTF8String],0,16);
                green =(float)strtoul([[string substringWithRange:NSMakeRange(1, 1)] UTF8String],0,16);
                blue =(float)strtoul([[string substringWithRange:NSMakeRange(2, 1)] UTF8String],0,16);
                break;
            case 6:
                red =(float)strtoul([[string substringWithRange:NSMakeRange(0, 2)] UTF8String],0,16);
                green =(float)strtoul([[string substringWithRange:NSMakeRange(2, 2)] UTF8String],0,16);
                blue =(float)strtoul([[string substringWithRange:NSMakeRange(4, 2)] UTF8String],0,16);
                break;
            case 8:
                opacity =(float)strtoul([[string substringWithRange:NSMakeRange(0, 2)] UTF8String],0,16)/255;
                red =(float)strtoul([[string substringWithRange:NSMakeRange(2, 2)] UTF8String],0,16);
                green =(float)strtoul([[string substringWithRange:NSMakeRange(4, 2)] UTF8String],0,16);
                blue =(float)strtoul([[string substringWithRange:NSMakeRange(6, 2)] UTF8String],0,16);
                break;
            default:
                break;
        }
    }else if([colorString hasPrefix:@"rgba"] == 1){
        NSString *string = [colorString substringWithRange:NSMakeRange(5, colorString.length-1)];
        NSArray *_arr = [string componentsSeparatedByString:NSLocalizedString(@",", nil)];
        red =[(NSString *)[_arr objectAtIndex:0] floatValue] ;
        green =[(NSString *)[_arr objectAtIndex:1] floatValue] ;
        blue =[(NSString *)[_arr objectAtIndex:2] floatValue] ;
        opacity =[(NSString *)[_arr objectAtIndex:3] floatValue] ;
        
    }else if([colorString hasPrefix:@"rgb"] == 1){
        NSString *string = [colorString substringWithRange:NSMakeRange(4, colorString.length-1)];
        NSArray *_arr = [string componentsSeparatedByString:NSLocalizedString(@",", nil)];
        red =[(NSString *)[_arr objectAtIndex:0] floatValue] ;
        green =[(NSString *)[_arr objectAtIndex:1] floatValue] ;
        blue =[(NSString *)[_arr objectAtIndex:2] floatValue] ;
    }
    return [UIColor colorWithRed:(float)(red/255.0f) green:(float)(green / 255.0f) blue:(float)(blue / 255.0f) alpha:opacity];
    
}

@end
