//
//  MyVideoView.m
//  m
//
//  Created by Spencer on 16/7/15.
//

#import "MyVideoView.h"

static NSString *const statusKeyPath = @"status";
static NSString *const loadedTimeRanges=@"loadedTimeRanges";
static NSString *const playbackLikelyToKeepUpKeyPath = @"playbackLikelyToKeepUp";
static NSString *const playbackBufferEmptyKeyPath = @"playbackBufferEmpty";
static NSString *const readyForDisplayKeyPath = @"readyForDisplay";
static NSString *const playbackRate = @"rate";

@implementation MyVideoView
{
  
  AVPlayer *_player;//播放器对象
  
  AVPlayerItem *_playerItem;
  BOOL _playerItemObserversSet;
  BOOL _playerBufferEmpty;
  AVPlayerLayer *_playerLayer;
  NSURL *_videoUrl;
  BOOL viewDidLoad;
  /*
 // Only override drawRect: if you perform custom drawing.
 // An empty implementation adversely affects performance during animation.
 - (void)drawRect:(CGRect)rect {
 // Drawing code
 }
 */
}
- (void)dealloc
{
  @try{
    [_player pause];
    [self removePlayerLayer];
    [self removePlayerItemObserver];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
  }@catch(NSException *e){
    NSLog(@"Exception: %@", e);
  }
  
}
- (id)initWithFrame:(CGRect)frame {
  self = [super initWithFrame:frame];
  if (self) {
    [self initSet];
  }
  return self;
}
-(void)drawRect:(CGRect)aRect{
  [self addPlayerLayer];
}
#pragma mark - 私有方法
-(void)setSrc:(NSString *)src{
  NSString*thePath=[[NSBundle mainBundle] pathForResource:@"start_film" ofType:@"mp4"];
  _videoUrl =[NSURL fileURLWithPath:thePath];
  [_player pause];
  [self setPlayerItemByURL:_videoUrl];
  [self replaceCurrentItemWithPlayerItem:_playerItem];
  [self applyModifiers];
}
-(void)initSet{
  _playInBackground = false;
  _muted = false;
  _volume = 1.0f;
  _paused = false;
  _repeat = YES;
  _autoPlay = YES;
  [self setPlayerItemByURL:[NSURL URLWithString:@""]];
  [self initPlayer];
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(applicationWillResignActive:)
                                               name:UIApplicationWillResignActiveNotification
                                             object:nil];
  
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(applicationDidEnterBackground:)
                                               name:UIApplicationDidEnterBackgroundNotification
                                             object:nil];
  
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(applicationWillEnterForeground:)
                                               name:UIApplicationWillEnterForegroundNotification
                                             object:nil];
  
}
- (void)setResizeMode:(NSString*)mode
{
  _playerLayer.videoGravity = mode;
  _resizeMode = mode;
}
-(void)didMoveToSuperview{
  NSLog(@"myvideoview didmovetosuperview");
  if(!viewDidLoad){
    viewDidLoad = YES;
    return;
  }
  @try{
    [_player pause];
    [self removePlayerLayer];
    [self removePlayerItemObserver];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
  }@catch(NSException *e){
    NSLog(@"Exception: %@", e);
  }
  viewDidLoad = NO;

}
- (void)setPlayInBackground:(BOOL)playInBackground
{
  _playInBackground = playInBackground;
}
- (void)setRate:(float)rate
{
  _rate = rate;
  [self applyModifiers];
}

- (void)setMuted:(BOOL)muted
{
  _muted = muted;
  [self applyModifiers];
}

- (void)setVolume:(float)volume
{
  _volume = volume;
  [self applyModifiers];
}
-(void)setPaused:(BOOL)paused{
//  if (paused) {
//    [_player pause];
//    [_player setRate:0.0];
//  } else {
//    [_player play];
//    [_player setRate:_rate];
//  }
  _paused = paused;
}
- (void)setRepeat:(BOOL)repeat {
  _repeat = repeat;
}
- (void)setAutoPlay:(BOOL)autoPlay{
  _autoPlay = autoPlay;
}
- (void)applyModifiers
{
  if (_muted) {
    [_player setVolume:0];
    [_player setMuted:YES];
  } else {
    [_player setVolume:_volume];
    [_player setMuted:NO];
  }
  [_player setRate:_rate];
  [self setResizeMode:_resizeMode];
  [self setRepeat:_repeat];
  [self setPaused:_paused];
}

-(void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context{
  AVPlayerItem *playerItem=object;
  if ([keyPath isEqualToString:@"status"]) {
    AVPlayerStatus status= [[change objectForKey:@"kind"] intValue];
    if(status==AVPlayerStatusReadyToPlay){
      NSLog(@"正在播放...，视频总长度:%.2f",CMTimeGetSeconds(playerItem.duration));
      if(_autoPlay){
        [self play];
      }
      [self attachListeners];
    }
  }else if([keyPath isEqualToString:@"loadedTimeRanges"]){
    NSArray *array=playerItem.loadedTimeRanges;
    CMTimeRange timeRange = [array.firstObject CMTimeRangeValue];//本次缓冲时间范围
    float startSeconds = CMTimeGetSeconds(timeRange.start);
    float durationSeconds = CMTimeGetSeconds(timeRange.duration);
    NSTimeInterval totalBuffer = startSeconds + durationSeconds;//缓冲总长度
    //
  }
}
-(AVPlayerItem *)getPlayerItemByURL:(NSURL *)url{
  AVPlayerItem *playerItem=[AVPlayerItem playerItemWithURL:url];
  return playerItem;
  
}
-(void)setPlayerItemByURL:(NSURL *)url{
  if(_playerItem){
    [self removePlayerItemObserver];
   }
  _playerItem =[self getPlayerItemByURL:url];
  [self addPlayerItemObserver];
}
-(void)addPlayerItemObserver{
  [_playerItem addObserver:self forKeyPath:statusKeyPath options:0 context:nil];
  [_playerItem addObserver:self forKeyPath:loadedTimeRanges options:0 context:nil];
  [_playerItem addObserver:self forKeyPath:playbackBufferEmptyKeyPath options:0 context:nil];
  [_playerItem addObserver:self forKeyPath:playbackLikelyToKeepUpKeyPath options:0 context:nil];
  [_player addPeriodicTimeObserverForInterval:CMTimeMake(1.0, 1.0) queue:dispatch_get_main_queue() usingBlock:^(CMTime time) {
    float current=CMTimeGetSeconds(time);
    float total=CMTimeGetSeconds([_playerItem duration]);
    NSLog(@"当前已经播放%.2fs.",current);
  }];
}
-(void)removePlayerItemObserver{
  [_playerItem removeObserver:self forKeyPath:statusKeyPath];
  [_playerItem removeObserver:self forKeyPath:loadedTimeRanges];
  [_playerItem removeObserver:self forKeyPath:playbackBufferEmptyKeyPath];
  [_playerItem removeObserver:self forKeyPath:playbackLikelyToKeepUpKeyPath];
}
-(void)initPlayer{
  _player = [AVPlayer playerWithPlayerItem:_playerItem];
}
-(void)replaceCurrentItemWithPlayerItem:(AVPlayerItem *)playerItem{
  [_player replaceCurrentItemWithPlayerItem:playerItem];
}
-(void)play{
  [_player play];
}
- (void)removePlayerLayer
{
  [_playerLayer removeFromSuperlayer];
  [_playerLayer removeObserver:self forKeyPath:readyForDisplayKeyPath];
  _playerLayer = nil;
}
-(void)addPlayerLayer{
  if( _player )
  {
    _playerLayer = [AVPlayerLayer playerLayerWithPlayer:_player];
    [_playerLayer addObserver:self forKeyPath:readyForDisplayKeyPath options:NSKeyValueObservingOptionNew context:nil];
    _playerLayer.frame = self.bounds;
    _playerLayer.needsDisplayOnBoundsChange = YES;
        // 6、设置拉伸模式
    _playerLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
    [self.layer addSublayer:_playerLayer];
    self.layer.needsDisplayOnBoundsChange = YES;
  }

}
- (void)applicationWillResignActive:(NSNotification *)notification
{
  if (_playInBackground  || _paused) return;
  
  [_player pause];
  [_player setRate:0.0];
}

- (void)applicationDidEnterBackground:(NSNotification *)notification
{
  if (_playInBackground) {
    [_playerLayer setPlayer:nil];
  }
}

- (void)applicationWillEnterForeground:(NSNotification *)notification
{
  [self applyModifiers];
  if (_playInBackground) {
    [_playerLayer setPlayer:_player];
  }
}
- (void)playerItemDidReachEnd:(NSNotification *)notification
{
  if (_repeat) {
    AVPlayerItem *item = [notification object];
    [item seekToTime:kCMTimeZero];
    [self play];
  }
}
- (void)playbackStalled:(NSNotification *)notification
{
  //_playbackStalled = YES;
}
- (void)attachListeners
{
  // listen for end of file
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(playerItemDidReachEnd:)
                                               name:AVPlayerItemDidPlayToEndTimeNotification
                                             object:[_player currentItem]];
  [[NSNotificationCenter defaultCenter] addObserver:self
                                           selector:@selector(playbackStalled:)
                                               name:AVPlayerItemPlaybackStalledNotification
                                             object:nil];
}
@end
