/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

#import "RCTAresWebViewManager.h"




@interface RCTAresWebViewManager ()

@end

@implementation RCTAresWebViewManager
{
  NSConditionLock *_shouldStartLoadLock;
  BOOL _shouldStartLoad;
}
@synthesize bridge = _bridge;


-(RCTBridge *)getBridge{
    return self.bridge;
}
- (void)setBridge:(RCTBridge *)bridge
{
    _bridge = bridge;
}
RCT_EXPORT_MODULE(RCTAresWebView)
- (UIView *)view
{
  AresWebView *webView = [[AresWebView alloc] init];
  return webView;
}

RCT_EXPORT_VIEW_PROPERTY(source, NSDictionary)
//RCT_REMAP_VIEW_PROPERTY(bounces, _webView.scrollView.bounces, BOOL)
//RCT_REMAP_VIEW_PROPERTY(scrollEnabled, _webView.scrollView.scrollEnabled, BOOL)
//RCT_REMAP_VIEW_PROPERTY(decelerationRate, _webView.scrollView.decelerationRate, CGFloat)
//RCT_EXPORT_VIEW_PROPERTY(scalesPageToFit, BOOL)
//RCT_EXPORT_VIEW_PROPERTY(injectedJavaScript, NSString)
//RCT_EXPORT_VIEW_PROPERTY(contentInset, UIEdgeInsets)
//RCT_EXPORT_VIEW_PROPERTY(automaticallyAdjustContentInsets, BOOL)
RCT_EXPORT_VIEW_PROPERTY(onProgressChangedKey, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onPageFinishedKey, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onPageStartedKey, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onShouldStartLoadWithRequestKey, RCTDirectEventBlock)
RCT_EXPORT_VIEW_PROPERTY(onInvokeKey, RCTDirectEventBlock)
//RCT_REMAP_VIEW_PROPERTY(allowsInlineMediaPlayback, _webView.allowsInlineMediaPlayback, BOOL)
//RCT_REMAP_VIEW_PROPERTY(mediaPlaybackRequiresUserAction, _webView.mediaPlaybackRequiresUserAction, BOOL)


RCT_EXPORT_METHOD(goBack:(nonnull NSNumber *)reactTag)
{
  [self.bridge.uiManager addUIBlock:^(__unused RCTUIManager *uiManager, NSDictionary<NSNumber *, AresWebView *> *viewRegistry) {
    AresWebView *view = viewRegistry[reactTag];
    if (![view isKindOfClass:[AresWebView class]]) {
      RCTLogError(@"Invalid view returned from registry, expecting RCTWebView, got: %@", view);
    } else {
        [view goBack];
    }
  }];
}

RCT_EXPORT_METHOD(goForward:(nonnull NSNumber *)reactTag)
{
  [self.bridge.uiManager addUIBlock:^(__unused RCTUIManager *uiManager, NSDictionary<NSNumber *, UIView *> *viewRegistry) {
    id view = viewRegistry[reactTag];
    if (![view isKindOfClass:[AresWebView class]]) {
      RCTLogError(@"Invalid view returned from registry, expecting RCTWebView, got: %@", view);
    } else {
        [view goForward];
    }
  }];
}

RCT_EXPORT_METHOD(reload:(nonnull NSNumber *)reactTag)
{
  [self.bridge.uiManager addUIBlock:^(__unused RCTUIManager *uiManager, NSDictionary<NSNumber *, AresWebView *> *viewRegistry) {
    AresWebView *view = viewRegistry[reactTag];
    if (![view isKindOfClass:[AresWebView class]]) {
      RCTLogError(@"Invalid view returned from registry, expecting RCTWebView, got: %@", view);
    } else {
        [view reload];
    }
  }];
}
RCT_EXPORT_METHOD(loadJs:(nonnull NSNumber *)reactTag url:(NSString *)jsString)
{
  [self.bridge.uiManager addUIBlock:^(__unused RCTUIManager *uiManager, NSDictionary<NSNumber *, AresWebView *> *viewRegistry) {
    AresWebView *view = viewRegistry[reactTag];
    if (![view isKindOfClass:[AresWebView class]]) {
      RCTLogError(@"Invalid view returned from registry, expecting RCTWebView, got: %@", view);
    } else {
        [view loadJs:jsString];
    }
  }];
}
RCT_EXPORT_METHOD(loadUrl:(nonnull NSNumber *)reactTag url:(NSString *)url)
{
  [self.bridge.uiManager addUIBlock:^(__unused RCTUIManager *uiManager, NSDictionary<NSNumber *, AresWebView *> *viewRegistry) {
    AresWebView *view = viewRegistry[reactTag];
    if (![view isKindOfClass:[AresWebView class]]) {
      RCTLogError(@"Invalid view returned from registry, expecting RCTWebView, got: %@", view);
    } else {
      [view loadUrl:url];
    }
  }];
}

RCT_EXPORT_METHOD(startLoadWithResult:(BOOL)result lockIdentifier:(NSInteger)lockIdentifier)
{
  if ([_shouldStartLoadLock tryLockWhenCondition:lockIdentifier]) {
    _shouldStartLoad = result;
    [_shouldStartLoadLock unlockWithCondition:0];
  } else {
    RCTLogWarn(@"startLoadWithResult invoked with invalid lockIdentifier: "
               "got %zd, expected %zd", lockIdentifier, _shouldStartLoadLock.condition);
  }
}

@end
