'use strict'
class HttpEntity {
    constructor(request, bundle) {
        this.request = request;
        this.bundle = bundle || {};
        this._httpTimeoutHandle;
        this.isSending = false;
    }
    _finish(result,callback) {
        if (this._httpTimeoutHandle !== undefined) {
            clearTimeout(this._httpTimeoutHandle);
        }
        this._httpTimeoutHandle = undefined;
        callback(result);
    }

    sendRequest() {
        if (this.isSending) {
            throw new Error('can \' send request while a request is sending.');
        }
        return new Promise((resolve, reject) => {
            if (typeof this.bundle.timeout == 'number') {
                this._httpTimeoutHandle = setTimeout(
                    () => this._finish({ code: 503, message: '请求超时', timeout: true }, reject),
                    this.bundle.timeout);
            }
            return fetch(this.request).then((response) => {
                this._finish(response, resolve);
            }, (response) => {
                this._finish(response, reject);
            });
        });

    }
}

export default HttpEntity;