/**
 * @filename - app/view/AutoSizeImage.js
 * @description - $(currentFile)
 */
'use strict'
import React, { Component } from 'react';
import {
    StyleSheet,
    View,
    Text,
    Image,
    Animated
} from 'react-native';
export default class AutoSizeImage extends Component {
    constructor(...props) {
        super(...props);
        this.state = {
            uri:'',
            height:0,
            width:0,
            heightWidth:new Animated.ValueXY({x:this.props.initialWidth,y:this.props.initialHeight})
        }

    }
    componentDidMount() {
        this._getSize();
    }
    componentWilMount() {

    }
    componentWillReceiveProps(nextProps) {
        this._getSize();
    }
    _getSize() {
        if(!this.props.uri||typeof this.props.uri!=='string'){
            return;
        }
        if(this.state.uri!==this.props.uri){
            this.state.uri = this.props.uri;
            Image.getSize(this.props.uri, (width, height) => {
                this.state.height = height;
                this.state.width = width;
                this._resizeImage();
            }, (error) => {
                this.state.height = this.state.initialHeight;
                this.state.width = this.state.initialWidth;
                this._resizeImage();
            });
        }else{
            this._resizeImage();
        }
        
    }
    _resizeImage(){
        var {width,height} = this.state;
        var x,y;
        if (width > this.props.maxWidth) {
            x = this.props.maxWidth;
            y = x / width * height;
        } else if (height > this.props.maxHeight) {
            y = this.props.maxHeight;
            x = x / height * width;
        } else {
            x = width;
            y = height;
        }
        this.state.heightWidth.setValue({x,y});
    }
    getWidth() {
        return this.state.width;
    }
    getHeight() {
        return this.state.height;
    }
    render() {
        if (!this.props.uri) {
            return null;
        }
        var width = this.state.heightWidth.x;
        var height = this.state.heightWidth.y;
        return (
            <Animated.Image
                onLayout = {this.props.onLayout}
                source={{ uri: this.props.uri }}
                style={[this.props.style, { height, width }]}
            />

        );
    }
}
AutoSizeImage.defaultProps = {
    maxHeight: Infinity,
    maxWidth: Infinity,
    initialHeight: 0,
    initialWidth: 0
}