/*
 * @file - app/view/components/DatePicker.js
 * @description - 日期
 */
'use strict';

import React, { Component } from 'react';

import {
    View,
    StyleSheet,
    TouchableOpacity,
    InteractionManager,
    PixelRatio,
    Text,
    dismissKeyboard,
    ViewPropTypes
} from 'react-native';
import PropTypes from 'prop-types';


import Picker from './Picker'

var dateNow = new Date();
var nowYear = dateNow.getFullYear();
var nowMonth = dateNow.getMonth() + 1;
var nowDay = dateNow.getDate();
function formatString(v) {
    if (String(v).length < 2) {
        v = '0' + v;
    }
    return String(v);
}
class DayPicker extends Component {
    constructor(...props) {
        super(...props);
        this.state = {
            day: formatString(nowDay),
            lower: 1,
            upper: 31
        }
        this._stateFromProps(this.props);
    }
    _stateFromProps(props) {
        if (props.day) {
            this.state.day = formatString(props.day);
        }
        if (props.upper) {
            this.state.upper = props.upper;
        }
        if (props.lower) {
            this.state.lower = props.lower;
        }
    }
    componentWillReceiveProps(nextProps) {
        this._stateFromProps(nextProps);
    }
    _onValueChange(value){
        this.props.onValueChange&&this.props.onValueChange(value);
        this.setState({ day: value });
    }
    _renderRow() {
        var rows = [];
        var lower = this.state.lower;
        var upper = this.state.upper;
        for (; lower <= upper; lower++) {
            rows.push(
                <Picker.Item key={'day' + lower} label ={lower + '日'} value={formatString(lower) } />
            );
        }
        return rows;
    }
    render() {
        return (
            <Picker
                style = {{ flex: 1 }}
                selectedValue={this.state.day}
                onValueChange={(value) => this._onValueChange(value) }>
                {this._renderRow() }
            </Picker>
        );
    }
}
class YearPicker extends Component {
    constructor(...props) {
        super(...props);
        this.state = {
            year: formatString(nowYear),
            lower: nowYear - 100,
            upper: nowYear + 100
        }
        this._stateFromProps(this.props);
    }
    _onValueChange(value){
        this.props.onValueChange&&this.props.onValueChange(value);
        this.setState({ year: value });
    }
    _stateFromProps(props) {
        if (props.year) {
            this.state.year = formatString(props.year);
        }
        if (props.upper) {
            this.state.upper = props.upper;
        }
        if (props.lower) {
            this.state.lower = props.lower;
        }
    }
    componentWillReceiveProps(nextProps) {
        this._stateFromProps(nextProps);
    }
    _renderRow() {
        var rows = [];
        var lower = this.state.lower;
        var upper = this.state.upper;
        for (; lower <= upper; lower++) {
            rows.push(
                <Picker.Item key={'year' + lower} label ={lower + '年'} value={formatString(lower)} />
            );
        }
        return rows;
    }
    render() {
        return (
            <Picker
                style = {{ flex: 1 }}
                selectedValue={this.state.year}
                onValueChange={(value) => this._onValueChange(value) }>
                {this._renderRow() }
            </Picker>
        );
    }
}
class MonthPicker extends Component {
    constructor(...props) {
        super(...props);
        this.state = {
            month: formatString(nowMonth),
            lower: 1,
            upper: 12
        }
        this._stateFromProps(this.props);
    }
    _stateFromProps(props) {
        if (props.month) {
            this.state.month = formatString(props.month);
        }
        if (props.upper) {
            this.state.upper = props.upper;
        }
        if (props.lower) {
            this.state.lower = props.lower;
        }
    }
    componentWillReceiveProps(nextProps) {
        this._stateFromProps(nextProps);
    }
    _renderRow() {
        var rows = [];
        var lower = this.state.lower;
        var upper = this.state.upper;
        for (; lower <= upper; lower++) {
            rows.push(
               <Picker.Item key={'month' + lower} label ={lower + '月'} value={formatString(lower)} />
            );
        }
        return rows;
    }
    _onValueChange(value){
        this.props.onValueChange&&this.props.onValueChange(value);
        this.setState({ month: value });
    }
    render() {
        return (
            <Picker
                style = {{ flex: 1 }}
                selectedValue={this.state.month}
                onValueChange={(value) => this._onValueChange(value) }>
                {this._renderRow() }
            </Picker>
        );
    }
}
export default class DatePicker extends Component {
    constructor(...props) {
        super(...props);
        var now = Date.now();

        this.state = {
            year:formatString(nowYear),
            day:formatString(nowDay),
            month:formatString(nowMonth),
            lower:undefined,
            upper:undefined,
            yearBound:{},
            dayBound:{},
            monthBound:{}
        }
        this._stateFromProps(this.props);
    }
    _stateFromProps(props){
        if(props.year){
            this.state.year = formatString(props.year);
        }
        if(props.day){
            this.state.day = formatString(props.day);
        }
        if(props.month){
            this.state.month = formatString(props.month);
        }
        if(props.lowerTime){
            this.state.lower = new Date(props.lowerTime);
        }
        if(props.upperTime){
            this.state.upper = new Date(props.upperTime);
        }
        this._resetYearBound();
        this._resetMonthBound();
        this._resetDayBound();
    }
    _resetDayBound(){
        var lower = 1,upper = new Date(this.state.year, this.state.month, 0).getDate();
        var yearBound = this.state.yearBound;
        var monthBound = this.state.monthBound;
        if(this.state.lower&&yearBound.lower&&yearBound.lower>=this.state.year&&monthBound.lower&&monthBound.lower>=this.state.month){
            lower = this.state.lower.getDate();
        }
        if(this.state.upper&&yearBound.upper&&yearBound.upper<=this.state.year&&monthBound.upper&&monthBound.upper<=this.state.month){
            upper = this.state.upper.getDate();
        }
        if(parseInt(this.state.day)<lower){
            this.state.day = formatString(lower);
        }else if(parseInt(this.state.day)>upper){
            this.state.day = formatString(upper);
        }
        this.state.dayBound =  {lower,upper}
    }
    _resetMonthBound(){
        var lower = 1,upper = 12;
        var yearBound = this.state.yearBound;
        if(yearBound.lower&&yearBound.lower>=this.state.year&&this.state.lower){
             lower = this.state.lower.getMonth()+1;
        }
        if(yearBound.upper&&yearBound.upper<=this.state.year&&this.state.upper){
            upper = this.state.upper.getMonth()+1;
        }
        if(parseInt(this.state.month)<lower){
            this.state.month = formatString(lower);
        }else if(parseInt(this.state.month)>upper){
            this.state.month = formatString(upper);
        }
        this.state.monthBound = {lower,upper}
    }
    _resetYearBound(){
        var lower,upper;
        if(this.state.lower){
            lower = this.state.lower.getFullYear();
        }else{
            lower = dateNow.getFullYear()-100;
        }
        if(this.state.upper){
            upper = this.state.upper.getFullYear();
        }else{
            upper = dateNow.getFullYear()+100;
        }
        this.state.yearBound = {lower,upper}
    }
    componentWillReceiveProps(nextProps) {
        this._stateFromProps(nextProps);
    }

    _onCancel() {
        this.props.onCancel && this.props.onCancel();
        InteractionManager.runAfterInteractions(() => {
            dismissKeyboard();
        });
    }

    _onSelected() {
        this.props.onSelected && this.props.onSelected([this.state.year, this.state.month, this.state.day]);
        InteractionManager.runAfterInteractions(() => {
            dismissKeyboard();
        });
    }

    render() {
        return (
            <View
                style = {[this.props.style, { flexDirection: 'column' }]}>
                <View style = {styles.header}>
                    <TouchableOpacity
                        style = {styles.btn}
                        activeOpacity = {1}
                        onPress = {() => this._onCancel() }>
                        <Text style = {styles.btnText}>取消</Text>
                    </TouchableOpacity>
                    <TouchableOpacity
                        style = {[styles.btn, { justifyContent: 'flex-end' }]}
                        activeOpacity = {1}
                        onPress = {() => this._onSelected() }>
                        <Text style = {styles.btnText}>完成</Text>
                    </TouchableOpacity>
                </View>
                <View  style = {styles.picker}>
                    <YearPicker
                        {...this.state.yearBound}
                        year = {this.state.year}
                        onValueChange = {(v)=>{
                            this.state.year = v;
                            this._resetMonthBound();
                            this._resetDayBound();
                            this.forceUpdate();
                        }}></YearPicker>
                    <MonthPicker
                        {...this.state.monthBound}
                        month = {this.state.month}
                        onValueChange = {(v)=>{
                            this.state.month = v;
                            this._resetDayBound();
                            this.forceUpdate();
                        }}></MonthPicker>
                    <DayPicker
                        {...this.state.dayBound}
                        day = {this.state.day}
                        onValueChange = {(v)=>this.state.day = v}></DayPicker>
                </View>
            </View>
        );
    }
}

DatePicker.propTypes = {
    ...ViewPropTypes,
    onCancel: PropTypes.func,
    onSelected: PropTypes.func,
    upperTime: PropTypes.number,
    lowerTime: PropTypes.number
};

const styles = StyleSheet.create({
    header: {
        height: 40,
        flexDirection: 'row',
        justifyContent: 'space-between',
        alignItems: 'center',
        backgroundColor: '#fff',
        borderTopWidth: 1/PixelRatio.get(),
        borderColor: 'rgb(224,224,224)',
    },
    picker: {
        flex: 1,
        flexDirection: 'row',
        alignItems: 'center',
        backgroundColor: '#fff',
        borderTopWidth: 1/PixelRatio.get(),
        borderColor: 'rgb(224,224,224)',
        height: 220,
    },
    btn: {
        width: 64 ,
        height: 40,
        alignItems: 'center',
        flexDirection: 'row',
        justifyContent: 'flex-start',
    },
    btnText: {
        color: '#000',
        fontSize: 14,
        marginHorizontal: 12,
    }
});
