/*
 * @file - app/view/components/IPicker.js
 * @description - Picker封装
 */
'use strict';
import React, { Component } from 'react';
import {
    View,
    StyleSheet,
    TouchableOpacity,
    Animated,
    InteractionManager,
    Text,
    Keyboard,
    PixelRatio,
    ViewPropTypes
} from 'react-native';
import PropTypes from 'prop-types';


import Picker from './Picker';

export default class IPicker extends Component {
    constructor(...props) {
        super(...props);
        this.state = {
            default: '',
            val: this.props.val || 'id',
            label: this.props.label || 'name',
        }
    }

    // 点取消
    _onCancel() {
        this.props.onCancel && this.props.onCancel();      
        InteractionManager.runAfterInteractions(()=>{
            Keyboard.dismiss();
        });
    }

    // 点完成
    _onSelected() {
        var _this = this;
        var currentVal = _this.state.default || _this.props.selectedVal || '1'; // 优先取picker change之后的值，其次取props传进来的，默认是1
        let item = _this.props.dataSource.find((v) => {
            return currentVal == v[this.state.val];
        }) || {};
        _this.props.onSelected && _this.props.onSelected(item);

        InteractionManager.runAfterInteractions(()=>{
            Keyboard.dismiss();
        });
    }

    render() {
        return (
            <View
                style = {[this.props.style, { flexDirection: 'column', position: 'absolute', left: 0, right: 0, bottom: 0, },]}>
                <View style = {styles.header}>
                    <TouchableOpacity
                        style = {styles.btn}
                        activeOpacity = {1}
                        onPress = {() => this._onCancel() }>
                        <Text style = {styles.btnText}>{this.props.cancelText || '取消'}</Text>
                    </TouchableOpacity>
                    <TouchableOpacity
                        style = {[styles.btn, { justifyContent: 'flex-end' }]}
                        activeOpacity = {1}
                        onPress = {() => this._onSelected() }>
                        <Text style = {styles.btnText}>{this.props.sureText || '完成'}</Text>
                    </TouchableOpacity>
                </View>
                <View style = {styles.picker}>
                    <Picker
                        style={styles.pickerItem}
                        selectedValue={(this.state.default || this.props.selectedVal || 1) + ''}
                        onValueChange={(value) => this.setState({ default: value }) }>
                        {
                            this.props.dataSource.map((v) => {
                                return (<Picker.Item key = {v[this.state.val]} label={v[this.state.label]} value={v[this.state.val]+''}/>);
                            })
                        }
                    </Picker>
                </View>
            </View>
        );
    }
}
IPicker.propTypes = {
    ...ViewPropTypes,
    onCancel: PropTypes.func,
    onSelected: PropTypes.func,
    dataSource: PropTypes.array,
    val: PropTypes.string,
    label: PropTypes.string,
};

IPicker.defaultProps = {
    dataSource: [],
    val: 'id',
    label: 'name',
};

const styles = StyleSheet.create({
    header: {
        height: 40,
        flexDirection: 'row',
        justifyContent: 'space-between',
        alignItems: 'center',
        backgroundColor:'#fff',
        borderTopWidth: 1/PixelRatio.get(),
        borderColor: 'rgb(224,224,224)',
    },
    picker: {
        flex: 1,
        flexDirection: 'row',
        alignItems: 'center',
        backgroundColor: '#fff',
        borderTopWidth: 1/PixelRatio.get(),
        borderColor: 'rgb(224,224,224)',
        height: 220,
    },
    pickerItem: {
        flex: 1,
    },
    btn: {
        width: 64,
        height: 20,
        alignItems: 'center',
        flexDirection: 'row',
        justifyContent: 'flex-start',
    },
    btnText: {
        color: '#000',
        fontSize: 14,
        marginHorizontal: 12,
    },
});