/*
 * @file - node_modules/react-native-router/src/Modal.js
 * @description - todo
 */
'use strict'
import React, { Component } from 'react';
import {
    StyleSheet,
    View,
    Text,
    Animated,
    Easing,
    Dimensions,
    InteractionManager
} from 'react-native';
import PropTypes from 'prop-types';
let window = Dimensions.get('window');
export default class Modal extends Component {
    constructor(...props) {
        super(...props);
        this.state = {

        }
        this.animated = new Animated.Value(0);
    }
    show() {
    }
    hide() {
        InteractionManager.runAfterInteractions(() => {
            if (this.props.animationType == 'none') {
                this.animated.setValue(0);
                this.props.onRequestClose && this.props.onRequestClose();
            } else {
                Animated.timing(this.animated, {
                    easing: Easing.ease,
                    toValue: 0,
                    duration: 200
                }).start(() => this.props.onRequestClose && this.props.onRequestClose());
            }
        })
    }

    componentDidMount() {
        InteractionManager.runAfterInteractions(() => {
            this.props.onDidMount && this.props.onDidMount(this);
            if (this.props.animationType == 'none') {
                this.animated.setValue(1);
                this.props.onShow && this.props.onShow();
            } else {
                Animated.timing(this.animated, {
                    easing: Easing.ease,
                    toValue: 1,
                    duration: 200
                }).start(() => this.props.onShow && this.props.onShow());
            }
        })
    }
    componentWillUnmount() {

    }

    render() {
        let opacity, translateY
        switch (this.props.animationType) {
            case 'fade':
                opacity = this.animated.interpolate({
                    inputRange: [0, 1],
                    outputRange: [0, 1]
                });
                translateY = 0;
                break;
            default:
                opacity = 1;
                translateY = this.animated.interpolate({
                    inputRange: [0, 1],
                    outputRange: [window.height, 0]
                });
        }
        return (
            <Animated.View
                children={this.props.children}
                style={[styles.base, { opacity, transform: [{ translateY }] }]} />
        );
    }
}

const styles = StyleSheet.create({
    base: {
        position: 'absolute',
        top: 0,
        left: 0,
        right: 0,
        bottom: 0,
        flexDirection: 'column'
    }
});
Modal.defaultProps = {
    animationType: 'slide',
    backgroundColor: 'transparent'
}
Modal.propTypes = {
    animationType: PropTypes.oneOf(['none', 'slide', 'fade'])
}