/*
 * @file - app/view/components/TextInput.js
 * @description - 输入框
 */
'use strict'
import React, {Component} from 'react';
import ReactNative, {
    View,
    TextInput,
    StyleSheet,
    Image,
    TouchableOpacity,
    Platform
} from 'react-native';
import PropTypes from 'prop-types';
const DELETE_ICON = require('./icon_del.png');

const IS_ANDROID =Platform.OS != 'ios';
class _TextInput extends Component {
    constructor(...props) {
        super(...props);
        this.state = {
            focused: false
        }
    }
    componentWillUnmount() {
        //clearTimeout(this.timeout);
    }
    setStyle(style){
        this.refs.view.setNativeProps({style});
    }
    _clearText() {
        this.refs.TextInput&&this.refs.TextInput.clear();
        this._onChangeText('');
        this._onChange({ nativeEvent: { text: '' } });
    }
    clear(){
        this._clearText();
    }
    _onFocus(e) {
        //clearTimeout(this.timeout);
        this.setState({
            focused: true
        });
        this.props.onFocus && this.props.onFocus(e);
    }
    _onBlur(e) {
        this.setState({
            focused: false
        });

        this.props.onBlur && this.props.onBlur(e);
    }
    _onChangeText(text) {
        this.props.onChangeText && this.props.onChangeText(text);
    }
    _onChange(e) {
        this.props.onChange && this.props.onChange(e);
    }
    render() {
        return (
            <View
                ref = 'view'
                style = {[this.props.style, styles.wrapper]}>
                <TextInput
                    {...this.props}
                    ref = 'TextInput'
                    underlineColorAndroid = 'transparent'
                    clearButtonMode = {this.props.showDelete ? 'while-editing' : 'never'}
                    style = {[{ textAlignVertical: this.props.multiline ? 'top' : 'center' }, this.props.textStyle, styles.baseInput,]}
                    onBlur = {(e) => this._onBlur(e) }
                    onChangeText = {(text) => this._onChangeText(text) }
                    onChange = {(e) => this._onChange(e) }
                    placeholderTextColor = {this.props.placeholderTextColor ? this.props.placeholderTextColor : 'rgba(0,0,0,0.26)'}
                    onFocus = {(e) => this._onFocus(e) }/>
                {IS_ANDROID && this.props.showDelete && (!!this.props.value) && this.state.focused && <TouchableOpacity
                    style = {styles.delBtn}
                    onPress = {() => this._clearText() }><Image style = {styles.delIcon} source = {DELETE_ICON}/></TouchableOpacity>}

            </View>
        );
    }
}

_TextInput.propTypes = {
    ...TextInput.propTypes,
    textStyle: TextInput.propTypes.style,
    showDelete: PropTypes.bool
};
_TextInput.defaultProps = {
    showDelete: true
}
const styles = StyleSheet.create({
    wrapper: {
        flexDirection: 'row',
    },
    baseInput: {
        flex: 1,
        backgroundColor: 'transparent',
        borderWidth: 0,
        borderColor: 'transparent',
        paddingHorizontal: 0,
        paddingVertical: !IS_ANDROID? 0 : 6,
    },
    delBtn: {
        flexDirection: 'row',
        justifyContent: 'center',
        alignItems: 'center',
        position: 'absolute',
        top: 0,
        bottom: 0,
        right: -5 ,
        paddingHorizontal: 8,
    },
    delIcon: {

    }
});
export default _TextInput;