/**
 * @description 自己封装的webview
 */
'use strict'

import React, { Component } from 'react';
import PropTypes from 'prop-types';
import ReactNative, { requireNativeComponent, View, UIManager, Platform,ViewPropTypes } from 'react-native';
import WebViewJsModule from './../jsmodules/WebViewJsModule';
var tag = 1;
const RCT_WEBVIEW_REF = 'RCT_WEBVIEW_REF';
const PROPS = ['onPageStarted', 'onPageFinished', 'onProgressChanged', 'onShouldStartLoadWithRequest', 'onInvoke'];
const IOS = Platform.OS == 'ios';
class WebView extends Component {
  constructor(...props) {
    super(...props);
    this.name = 'WebView';
    this.tag = tag++;
    this.methods = {}
    this._registerMethods();
  }

  loadUrl(url) {
    this._dispatchCommand(
      'loadUrl',
      [url]
    );
  }
  reload() {
    this._dispatchCommand(
      'reload',
      null
    );
  }
  goBack() {
    this._dispatchCommand(
      'goBack',
      null
    );
  }
  loadJs(jsString) {
    this._dispatchCommand(
      'loadJs',
      [jsString]
    );
  }
  _dispatchCommand(command, args) {
    UIManager.dispatchViewManagerCommand(
      this._getWebViewHandle(),
      UIManager.RCTWebViewLL.Commands[command],
      args
    );
  }
  _getWebViewHandle() {
    return ReactNative.findNodeHandle(this.refs[RCT_WEBVIEW_REF]);
  };


  componentDidMount() {

  }

  componentWillMount() {

  }
  componentWillUnmount() {
    this._releaseMethods();
  }
  _registerMethods() {
    PROPS.forEach((key) => {
      if (IOS) {
        this.methods[key + 'Key'] = (event) => {
          this['_' + key](...event.nativeEvent.params);
        };
        return;
      }
      this.methods[key + 'Key'] = key + this.tag;
      WebViewJsModule.register(key + this.tag, (...args) => {
        this['_' + key](...args);
      })
    })
  }
  _releaseMethods() {
    if (IOS) {
      return;
    }
    PROPS.forEach((key) => {
      WebViewJsModule.release(key + this.tag)
    })
  }
  _onPageStarted(...args) {
    this.props.onPageStarted && this.props.onPageStarted(...args);
  }
  _onPageFinished(...args) {
    this.props.onPageFinished && this.props.onPageFinished(...args);
  }
  _onInvoke(...args) {
    this.props.onInvoke && this.props.onInvoke(...args);
  }
  _onShouldStartLoadWithRequest(...args) {
    this.props.onShouldStartLoadWithRequest && this.props.onShouldStartLoadWithRequest(...args);
  }
  _onProgressChanged(...args) {
    this.props.onProgressChanged && this.props.onProgressChanged(...args);
  }
  render() {
    var methodsKey = {};
    PROPS.forEach((key) => {
      methodsKey[key]
    })
    var props = {
      ...this.props,
      ...this.methods
    }

    return <RCTAresWebView ref={RCT_WEBVIEW_REF} {...props} />
  }
}
WebView.propTypes = {
  source: PropTypes.object,
  onPageStarted: PropTypes.func,
  onPageFinished: PropTypes.func,
  onProgressChanged: PropTypes.func,
  onShouldStartLoadWithRequest: PropTypes.func,
  onInvoke: PropTypes.func,
  ...ViewPropTypes // include the default view properties
}


var RCTAresWebView = requireNativeComponent('RCTAresWebView', WebView, {
  nativeOnly: {
    onPageStartedKey: true,
    onPageFinishedKey: true,
    onProgressChangedKey: true,
    onShouldStartLoadWithRequestKey: true,
    onInvokeKey: true,
  }
});

export default WebView;